
#include <cmath>
#include <iostream>
#include <GL/gl.h>
#include <GL/glui.h>

using std::cos;
using std::sin;
using std::sqrt;
using std::cout;
using std::endl;


#define	TESSNUM		32
#define PIVAL		0.196349541
#define	CYLHEIGHT	2.0

GLfloat gamb[] = { 0.1, 0.1, 0.1, 1.0 };

GLfloat pos0[] = { -13.0, 5.0, 0.0, 1.0 };
GLfloat amb0[] = { 0.0, 0.05, 0.05, 1.0 };
GLfloat dif0[] = { 1.0, 0.0, 0.0, 1.0 };
GLfloat spec0[] = { 1.0, 1.0, 1.0, 1.0 };

GLfloat pos1[] = { 5.0, 5.0, -5.0, 1.0 };
GLfloat amb1[] = { 0.05, 0.0, 0.05, 1.0 };
GLfloat dif1[] = { 0.0, 1.0, 0.0, 1.0 };
GLfloat spec1[] = { 1.0, 1.0, 1.0, 1.0 };

GLfloat pos2[] = { -5.0, 5.0, -5.0, 1.0 };
GLfloat amb2[] = { 0.05, 0.05, 0.0, 1.0 };
GLfloat dif2[] = { 0.0, 0.0, 1.0, 1.0 };
GLfloat spec2[] = { 1.0, 1.0, 1.0, 1.0 };

GLfloat mamb[] = { 0.05, 0.05, 0.05, 1.0 };
GLfloat mdif[] = { 0.25, 0.5, 0.75, 1.0 };
GLfloat mspec[] = { 0.95, 0.85, 0.75, 1.0 };

GLfloat mreset[] = { 0.0f, 0.0f, 0.0f, 1.0f};

int bLighting = 0;
int bLight0 = 0;
int bLight1 = 0; 
int bLight2 = 0;

int bDiffuse = 1;
int bSpecular = 0;
int bAmbient = 1;
int bGlobalAmbient = 0;
int bSmooth = 0;

float shininess = 30.0f;
float rotMat[16];

GLuint cylinderList = 0;

void buildUI(int mainWin)
{
	GLUI* uiWindow = GLUI_Master.create_glui("Lighting controls");

	GLUI_Panel *lightingPanel = uiWindow->add_panel("Lighting");

	uiWindow->add_checkbox_to_panel(lightingPanel, "Enabled", &bLighting);
	uiWindow->add_checkbox_to_panel(lightingPanel, "Light 0", &bLight0);
	uiWindow->add_checkbox_to_panel(lightingPanel, "Light 1", &bLight1);
	uiWindow->add_checkbox_to_panel(lightingPanel, "Light 2", &bLight2);

	GLUI_Panel *matPanel = uiWindow->add_panel("Material");

	uiWindow->add_checkbox_to_panel(matPanel, "Ambient term", &bAmbient);
	uiWindow->add_checkbox_to_panel(matPanel, "Diffuse term", &bDiffuse);
	uiWindow->add_checkbox_to_panel(matPanel, "Specular term", &bSpecular);

	GLUI_Spinner * spin = uiWindow->add_spinner_to_panel( matPanel
														, "Shininess"
														, GLUI_SPINNER_FLOAT
														, &shininess
													);

	spin->set_float_limits(0.5f, 100.0f);
	spin->set_speed(0.5f);

	uiWindow->add_checkbox("Global ambient term", &bGlobalAmbient);
	uiWindow->add_checkbox("Smooth shading", &bSmooth);

	GLUI_Rotation *rot = uiWindow->add_rotation("Orientation", rotMat);
	rot->reset();

	uiWindow->set_main_gfx_window(mainWin);
}

void genDisplayList()
{
	cylinderList = glGenLists(1);

	if(cylinderList != 0)
	{
		float inv = 1.0/TESSNUM;
		float forward, rev;

		glNewList(cylinderList, GL_COMPILE);

		glBegin(GL_TRIANGLE_STRIP);
		
		int i;
		for(i = 0; i < TESSNUM; ++i)
		{
			forward = i*inv;
			rev = 1.0-forward;

			glColor3f(rev, rev, 1.0);
			glNormal3d(cos(i*PIVAL), sin(i*PIVAL), 0);
			glVertex3d(cos(i*PIVAL), sin(i*PIVAL), -CYLHEIGHT);

			glColor3f(rev, forward, 0);
			glNormal3d(cos(i*PIVAL), sin(i*PIVAL), 0);
			glVertex3d(cos(i*PIVAL), sin(i*PIVAL), CYLHEIGHT);
		}

		// close the cylinder.
		glColor3f(0.0, 0.0, 1.0);
		glNormal3d(cos(i*PIVAL), sin(i*PIVAL), 0);
		glVertex3d(cos(i*PIVAL), sin(i*PIVAL), -CYLHEIGHT);

		glColor3f(0.0, 1.0, 0.0);
		glNormal3d(cos(i*PIVAL), sin(i*PIVAL), 0);
		glVertex3d(cos(i*PIVAL), sin(i*PIVAL), CYLHEIGHT);

		glEnd();

		glEndList();
	}
	else
	{
		cout << "Error generating display list." << endl;
	}
}

void drawCylinder()
{
	glCallList(cylinderList);
}

void glInit()
{
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);
	glClearDepth(100.0);
	glFrontFace(GL_CW);

	// setup lights
	glLightfv(GL_LIGHT0, GL_POSITION, pos0);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, dif0);
	glLightfv(GL_LIGHT0, GL_SPECULAR, spec0);

	glLightfv(GL_LIGHT1, GL_POSITION, pos1);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, dif1);
	glLightfv(GL_LIGHT1, GL_SPECULAR, spec1);

	glLightfv(GL_LIGHT2, GL_POSITION, pos2);
	glLightfv(GL_LIGHT2, GL_DIFFUSE, dif2);
	glLightfv(GL_LIGHT2, GL_SPECULAR, spec2);


	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, 1.0, 10.0, 100.0);

	glMatrixMode(GL_MODELVIEW);

	genDisplayList();
}

void reshape(int x, int y)
{
	glViewport(0, 0, x, y);
}

void display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	gluLookAt(-13, 0, 0, 0, 0, 0, 0, 1, 0);
	glMultMatrixf(rotMat);


	// shading
	if(bSmooth)	glShadeModel(GL_SMOOTH);
	else		glShadeModel(GL_FLAT);

	// lighting
	if(bLighting)	glEnable(GL_LIGHTING);
	else			glDisable(GL_LIGHTING);

	// individual lights
	if(bLight0)		glEnable(GL_LIGHT0);
	else			glDisable(GL_LIGHT0);

	if(bLight1)		glEnable(GL_LIGHT1);
	else			glDisable(GL_LIGHT1);

	if(bLight2)		glEnable(GL_LIGHT2);
	else			glDisable(GL_LIGHT2);

	// material terms
	if(bDiffuse)	glMaterialfv(GL_FRONT, GL_DIFFUSE, mdif);
	else			glMaterialfv(GL_FRONT, GL_DIFFUSE, mreset);

	if(bSpecular)	glMaterialfv(GL_FRONT, GL_SPECULAR, mspec);
	else			glMaterialfv(GL_FRONT, GL_SPECULAR, mreset);

	if(bAmbient)	glMaterialfv(GL_FRONT, GL_AMBIENT, mamb);
	else			glMaterialfv(GL_FRONT, GL_AMBIENT, mreset);

	glMaterialf(GL_FRONT, GL_SHININESS, shininess);

	if(bGlobalAmbient)
	{
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, gamb);
	}
	else
	{
		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, mreset);
	}

	drawCylinder();

	glutSwapBuffers();
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

	glutInitWindowPosition(200, 200);
	glutInitWindowSize(500, 500);

	int win = glutCreateWindow("Lighting Demo");

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);

	GLUI_Master.set_glutIdleFunc(NULL);

	buildUI(win);

	glInit();

	glutMainLoop();

	return 0;
}
